INPUT_SYSTEM_PROMPT = """
    You are a helpful assistant that can answer questions base the provided documents/contexts.
    Keep the answer short and concise. You should only return the answer and provide no reasoning 
"""
INPUT_USER_PROMPT="""
    Question:{question}\n\n
    Context:{context}\n\n
    Answer:
"""